/**
 *
 * 当前应用queen5接受的消息处理。
 *   目前文件依赖 设计为 当前文件 依赖于modules/ediotor/controllers下的文件，这样避免share页面打包
 *   当前模块，导致nats.ws文件被引用，引发 部分手机 share页面打不开的bug
 */

import { queenApi } from "queenjs";

import { AppMsgRecvController } from "@queenjs/assets";
import { nanoid } from "nanoid";
import { useCtx } from "../ctx";
import SaveForm from "./components/SaveForm";
export class AppScreenMsgController extends AppMsgRecvController {
  screenModule: any;
  constructor() {
    super();
  }

  initScreen() {
    this.clearListeners();
    this.addProductListener("添加模型", async (data) => {
      const { deviceCtrl } = useCtx();
      console.log("添加模型", data);
      if (data.payload.products.length != 1) return false;

      const config = (await queenApi.dialog(
        <SaveForm
          thumbnail={data.payload.products[0].thumbnail.url}
          name={data.payload.products[0].name}
        />,
        {
          title: "添加模型",
          width: "460px",
          maskClosable: false,
        }
      )) as any;
      console.log("config=>", config);

      const id = nanoid();
      let path = `/assets/${id}`;

      const source = data.payload;
      await this.downloadProductAssetFromHttp(path + "/", source);

      let thumbnail = source.products[0].thumbnail;
      if (thumbnail?.url) {
        const thumb = this.loadfile(`/assets/${id}/`, thumbnail.url);
        await thumb?.promise;
        source.products[0].thumbnail.url = thumb?.file;
      }

      const configFile = JSON.stringify(source);
      await deviceCtrl.WriteFileText(
        `${deviceCtrl.appDataDir}/screen/assets/${id}/${id}.json`,
        configFile
      );
      const currentCategory = config.category;
      const assetsJson = await deviceCtrl.ReadFileText(
        `${deviceCtrl.appDataDir}/screen/assets_${currentCategory}.json`
      );
      if (assetsJson.text) {
        const assets = JSON.parse(assetsJson.text);
        const assetItem = {
          _id: id,
          name: config.name,
          thumbnail: thumbnail.url,
          type: "3D",
          source: `/assets/${id}/${id}.json`,
        };
        assets.push(assetItem);
        await deviceCtrl.WriteFileText(
          `${deviceCtrl.appDataDir}/screen/assets_${currentCategory}.json`,
          JSON.stringify(assets)
        );
      }
      this.screenModule.actions.getAssetList();
      return true;
    });

    this.emitChange();
  }

  loadfile(dir: string, url: string) {
    if (!url) return;
    const { deviceCtrl } = useCtx();
    let names = url.split("/");
    let name = names[names.length - 1];
    const screenDir = deviceCtrl.appDataDir + "/screen";
    const filepath = screenDir + dir + name;
    return {
      file: dir + name,
      promise: new Promise((resolve) => {
        deviceCtrl.DownloadFile(url, filepath).then((ok) => {
          setTimeout(() => {
            resolve({ name: name, ok: ok });
          }, 0);
        });
      }),
    };
  }
  async downloadProductAssetFromHttp(
    path: string,
    source: { mats: any[]; geoms: any[]; products: any[] }
  ) {
    queenApi.showLoading("资源下载中");
    let files: any = [];

    this.downGeomsHttp(path, source.geoms, files);

    this.downMatsHttp(path, source.mats, files);

    const result = await Promise.all(files);
    queenApi.hideLoading();
    const failRes = result.filter((e) => {
      return e.ok == false;
    });
    console.log(result, failRes);

    return source;
  }
  downGeomsHttp(path: string, geoms: any[], files: any[]) {
    geoms.map((geom) => {
      const osgjsUrl = geom.osgjs.url;
      let osjsf = this.loadfile(path, osgjsUrl);
      if (osjsf) {
        files.push(osjsf?.promise);
        geom.osgjs.url = osjsf?.file;
      }

      const osgbinUrl = osgjsUrl.substring(0, osgjsUrl.length - 5) + "bin";
      let binf = this.loadfile(path, osgbinUrl);
      if (binf) {
        files.push(binf?.promise);
      }
      if (geom?.shadow?.url) {
        let shadowf = this.loadfile(path, geom.shadow.url);
        if (shadowf) {
          files.push(shadowf.promise);
          geom.shadow.url = shadowf.file;
        }
      }
    });
  }
  downMatsHttp(path: string, mats: any[], files: any[]) {
    //材质处理
    mats.forEach((mItem) => {
      const abf = this.loadfile(path, mItem.albedo.texture?.url);
      //base
      if (abf) {
        files.push(abf?.promise);
        mItem.albedo.texture.url = abf.file;
      }
      //法线
      if (mItem?.normal?.texture) {
        const norf = this.loadfile(path, mItem.normal.texture?.url);
        if (norf) {
          files.push(norf?.promise);
          mItem.normal.texture.url = norf.file;
        }
      }
      //粗糙度
      if (mItem?.roughness?.texture) {
        const rouf = this.loadfile(path, mItem.roughness.texture?.url);
        if (rouf) {
          files.push(rouf?.promise);
          mItem.roughness.texture.url = rouf.file;
        }
      }
      //金属度
      if (mItem?.metalness?.texture) {
        const metaf = this.loadfile(path, mItem.metalness.texture?.url);
        if (metaf) {
          files.push(metaf?.promise);
          mItem.metalness.texture.url = metaf.file;
        }
      }
      if (mItem?.opacity?.texture) {
        const opaf = this.loadfile(path, mItem.opacity.texture?.url);
        if (opaf) {
          files.push(opaf?.promise);
          mItem.opacity.texture.url = opaf.file;
        }
      }
    });
  }
}
